@@ -1,25 +1,14 @@
 import asyncio
 import itertools
-import time
 
 import pytest
 import trio
-import curio
 
 import pynng
 
 addr = "inproc://test-addr"
 
 
-@pytest.mark.curio
-async def test_arecv_asend_curio():
-    with pynng.Pair0(listen=addr, recv_timeout=1000) as listener, pynng.Pair0(
-        dial=addr
-    ) as dialer:
-        await dialer.asend(b"hello there buddy")
-        assert (await listener.arecv()) == b"hello there buddy"
-
-
 @pytest.mark.trio
 async def test_arecv_asend_asyncio():
     with pynng.Pair0(listen=addr, recv_timeout=1000) as listener, pynng.Pair0(
@@ -38,14 +27,6 @@ async def test_asend_arecv_trio():
         assert (await listener.arecv()) == b"hello there"
 
 
-@pytest.mark.curio
-async def test_arecv_curio_cancel():
-    with pynng.Pair0(listen=addr, recv_timeout=5000) as p0:
-        with pytest.raises(curio.CancelledError):
-            async with curio.timeout_after(0.001):
-                await p0.arecv()
-
-
 @pytest.mark.trio
 async def test_arecv_trio_cancel():
     with pynng.Pair0(listen=addr, recv_timeout=5000) as p0:
@@ -69,13 +50,6 @@ async def test_arecv_asyncio_cancel():
             await asyncio.gather(fut, cancel_soon(fut))
 
 
-@pytest.mark.curio
-async def test_asend_curio_send_timeout():
-    with pytest.raises(pynng.exceptions.Timeout):
-        with pynng.Pair0(listen=addr, send_timeout=1) as p0:
-            await p0.asend(b"foo")
-
-
 @pytest.mark.asyncio
 async def test_asend_asyncio_send_timeout():
     with pytest.raises(pynng.exceptions.Timeout):
